@@ -6,7 +6,7 @@ from datetime import datetime, timedelta
 from logging import Logger
 from random import random
 from time import sleep
-from typing import List, Deque, Callable
+from typing import List, Deque, Callable, Optional
 
 import psutil
 import pytz
@@ -57,7 +57,6 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
     WORKER_TASK_TIMEOUT: int = None  # Example 60
 
     DEDUPE_LOOK_AHEAD_MIN_ROWS = 100000
-    DEDUPE_PERIOD_SECONDS: int = 10
 
     _logger: Logger = None
     _QueueDeclarative: ACIProcessorQueueTupleABC = None
@@ -77,8 +76,6 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
         self._isProcessorEnabledCallable = isProcessorEnabledCallable
 
         self._pollLoopingCall = task.LoopingCall(self._poll)
-        self._nextDedupeTime = datetime.now(pytz.utc) \
-                               + self._randomiseTimeDelta(self.DEDUPE_PERIOD_SECONDS)
         self._nextVacuumTime = datetime.now(pytz.utc) \
                                + self._randomiseTimeDelta(self.VACUUM_PERIOD_SECONDS)
         self._queueCount = 0
@@ -101,7 +98,6 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
         assert self._QueueDeclarative, "ACI, _QueueDeclarative is missing"
         assert self.DEDUPE_LOOK_AHEAD_MIN_ROWS, \
             "ACI, DEDUPE_LOOK_AHEAD_MIN_ROWS is missing"
-        assert self.DEDUPE_PERIOD_SECONDS, "ACI, DEDUPE_PERIOD_SECONDS is missing"
         assert self._VacuumDeclaratives, "ACI, _VacuumDeclaratives is missing"
         assert self.VACUUM_PERIOD_SECONDS, "ACI, VACUUM_PERIOD_SECONDS is missing"
 
@@ -113,6 +109,9 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
     def isBusy(self) -> bool:
         return self._queueCount != 0
 
+    def isQueueEmpty(self) -> bool:
+        return self._queueCount == 0
+
     def start(self):
         self._processorStatusNotifier.setProcessorStatus(True, self._queueCount)
         d = self._pollLoopingCall.start(self.POLL_PERIOD_SECONDS, now=False)
@@ -185,9 +184,6 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
 
         self._processorStatusNotifier.setProcessorStatus(True, self._queueCount)
 
-        # Wait for the dedupe to complete
-        yield self._dedupeQueue()
-
         # Vacuum in the background
         d = self._vacuumTables()
         d.addErrback(vortexLogFailure, self._logger, consumeError=True)
@@ -268,11 +264,18 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
 
     @inlineCallbacks
     def _fetchBlocks(self) -> List[ACIProcessorQueueBlockItem]:
+
+        dedupSql = self._dedupeQueueSql(self._lastFetchedId,
+                                        self.DEDUPE_LOOK_AHEAD_MIN_ROWS) \
+            if self._lastFetchedId else None
+
         rawBlocks = yield runPyInPg(self._logger,
                                     self._dbSessionCreator,
                                     self._fetchBlocksInPg,
+                                    None,
                                     list(self._queueIdsInBuffer),
-                                    self._queueCount)
+                                    self._queueCount,
+                                    dedupSql)
 
         blocks = []
         for raw in rawBlocks:
@@ -288,7 +291,13 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
     @classmethod
     def _fetchBlocksInPg(cls, plpy,
                          queueIdsInBuffer: List[int],
-                         queueCount: int) -> List[ACIProcessorQueueBlockItem]:
+                         queueCount: int,
+                         dedupSql: Optional[str]) -> List[ACIProcessorQueueBlockItem]:
+        # ---------------
+        # Deduplicate the queue before we fetch more
+        if dedupSql:
+            plpy.execute(dedupSql)
+
         # ---------------
         # Prepare the input data
         queueIdsInBuffer = set(queueIdsInBuffer)
@@ -368,52 +377,6 @@ class ACIProcessorQueueControllerABC(metaclass=ABCMeta):
 
         return queueBlocks
 
-    # ---------------
-    # Deduplicate methods
-
-    def _dedupeQueue(self):
-        if datetime.now(pytz.utc) < self._nextDedupeTime:
-            return
-
-        self._nextDedupeTime = datetime.now(pytz.utc) \
-                               + self._randomiseTimeDelta(self.DEDUPE_PERIOD_SECONDS)
-
-        return deferToThreadWrapWithLogger(self._logger) \
-            (self._dedupeQueueWrapped) \
-            ()
-
-    def _dedupeQueueWrapped(self):
-        """ Deduplicate Queue
-
-        This method will look ahead and deduplicate the queue before this class loads
-        up the data.
-
-        This query will take approximately 32ms for a queue of 10,000
-
-        """
-        if not self._lastFetchedId:
-            return
-
-        dedupeLimit = self.DEDUPE_LOOK_AHEAD_MIN_ROWS
-        sql = self._dedupeQueueSql(self._lastFetchedId, dedupeLimit)
-        if not sql:
-            return
-
-        self._logger.debug("Duplicating queue for %s",
-                           self._QueueDeclarative.tupleType())
-
-        session = self._dbSessionCreator()
-        try:
-            session.execute(sql)
-            session.commit()
-
-        except Exception as e:
-            self._logger.error("Deduplication failed")
-            self._logger.exception(e)
-
-        finally:
-            session.close()
-
     @abstractmethod
     def _dedupeQueueSql(self, lastFetchedId: int, dedupeLimit: int):
         """ Deduplicate Queue SQL
